// Spaceship demo, close to white triangle

// This demo demonstrates the difference between orders of matrices,
// more exactly rotation and translation.
// Edit the code in display() to see different versions.

// The demo does explicit buffer handling.
// This version uses glDrawArrays.

#include "GL_utilities.h"
#include <math.h>
#include "MicroGlut.h"
// uses framework Cocoa
// uses framework OpenGL
#include "VectorUtils3.h"

// Globals
// Data would normally be read from files
GLfloat vertices[] = {   0, 0,
						-1, 1,
						2, 0,
						0, 0,
						2, 0,
						-1, -1
					};

// vertex array object
unsigned int vertexArrayObjID;
GLuint program;

void init(void)
{
	// two vertex buffer objects, used for uploading the
	unsigned int vertexBufferObjID;

	dumpInfo();

	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	printError("GL inits");

	// Load and compile shader
	program = loadShaders("spaceship.vert", "minimal.frag");
	printError("init shader");
	
	// Upload geometry to the GPU:
	
	// Allocate and activate Vertex Array Object
	glGenVertexArrays(1, &vertexArrayObjID);
	glBindVertexArray(vertexArrayObjID);
	// Allocate Vertex Buffer Objects
	glGenBuffers(1, &vertexBufferObjID);
	
	// VBO for vertex data
	glBindBuffer(GL_ARRAY_BUFFER, vertexBufferObjID);
	glBufferData(GL_ARRAY_BUFFER, 12*sizeof(GLfloat), vertices, GL_STATIC_DRAW);
	glVertexAttribPointer(glGetAttribLocation(program, "in_Position"), 2, GL_FLOAT, GL_FALSE, 0, 0); 
	glEnableVertexAttribArray(glGetAttribLocation(program, "in_Position"));

	// End of upload of geometry
	printError("init arrays");
}

float a = 0.0;

void display(void)
{
	printError("pre display");

	a += 0.1;
	
	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
//	mat4 m = Rz(a);
//	mat4 m = Mult(S(0.25, 0.25, 1), Rz(a));
	mat4 m = Mult(Mult(Rz(a), S(0.25, 0.25, 1)), T(0, -2.5, 0));
//	mat4 m = Mult(Mult(T(0, -0.5, 0), S(0.25, 0.25, 1)), Rz(a));
	glUniformMatrix4fv(glGetUniformLocation(program, "m"), 1, GL_TRUE, m.m);

	glBindVertexArray(vertexArrayObjID);	// Select VAO
	glDrawArrays(GL_TRIANGLES, 0, 6);	// draw object

	printError("display");
	
	glutSwapBuffers();
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutCreateWindow ("Spaceship");
	glutDisplayFunc(display); 
	glutRepeatingTimer(20);
	init ();
	glutMainLoop();
	return 0;
}
